//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by resource.rc
//
#define IDC_MYICON                      2
#define IDOK2                           3
#define IDD_DIALOG1                     101
#define IDS_STATUS                      101
#define IDD_DIALOG_SERVER               101
#define IDI_ICON1                       102
#define IDR_MENU1                       103
#define IDD_ABOUTBOX                    103
#define ID_FILE_EXIT                    104
#define ID_HELP_ABOUT                   105
#define IDD_DIALOG2                     106
#define IDD_DIALOG_SSERVER              106
#define ID_BOB_A                        108
#define ID_BOB_B                        109
#define ID_BOB_C                        110
#define ID_PLAYER_KICK                  111
#define IDD_DIALOG3                     113
#define IDD_DIALOG_SENV                 113
#define IDD_DIALOG4                     114
#define IDD_DIALOG_SECO                 114
#define IDD_DIALOG5                     115
#define IDD_DIALOG_SLAND                115
#define ID_SETTINGS_VIEWASEATTRIBUTES   116
#define ID_SETTINGS_CHANGESETTINGS      117
#define IDD_SAYBOX                      118
#define IDD_TALKDIALOG                  118
#define ID_PLAYER_TALKTOALL             119
#define ID_PLAYER_REFRESHINFO           120
#define ID_SETTINGS_SAVESETTINGS        121
#define IDD_DIALOG6                     122
#define IDD_DIALOG_SPLAYERS             122
#define IDB_BITMAP1                     134
#define IDD_LAUNCHER_DIALOG             202
#define IDS_APP_TITLE                   203
#define IDM_ABOUT                       204
#define IDM_EXIT                        205
#define IDC_LAUNCHER                    209
#define IDR_MAINFRAME                   228
#define ID_FILE_RESTOREDEFAULTS         231
#define ID_FILE_SAVEASDEFAULTS          232
#define IDD_LAUNCHERS_DIALOG            239
#define IDD_LAUNCHERFLAG_DIALOG         239
#define IDD_LAUNCHERNET_DIALOG          240
#define IDB_BITMAP2                     242
#define IDD_DIALOG_SMAIN                243
#define IDC_EDIT1                       1001
#define IDC_SERVER_NAME                 1001
#define IDC_EDIT_TALK                   1001
#define IDC_LIST1                       1002
#define IDC_LIST2                       1003
#define IDC_LIST3                       1004
#define IDC_LIST4                       1005
#define IDC_SERVER_MAX_PLAYERS          1010
#define IDC_SERVER_MIN_PLAYERS          1011
#define IDC_SERVER_ROUNDS               1012
#define IDC_INTEREST                    1012
#define IDC_SERVER_PORT                 1013
#define IDC_BUYONROUND                  1013
#define IDC_SERVER_SIMUL                1014
#define IDC_MONEYPERHIT                 1014
#define IDC_WAIT_TIME                   1015
#define IDC_MONEYPERROUND               1015
#define IDC_COMBO_FORCE                 1016
#define IDC_SHOT_TIME                   1016
#define IDC_COMBO_WINDCHANGES           1017
#define IDC_SHOT_TIME2                  1017
#define IDC_IDLE_TIME                   1017
#define IDC_COMBO_WALLTYPE              1018
#define IDC_STARTMONEY                  1018
#define IDC_COMBO_WEAPONSCALE           1019
#define IDC_LANDCOVERAGE                1019
#define IDC_LANDHEIGHT                  1020
#define IDC_COMBO_ARMSLEVEL             1020
#define IDC_COMBO_PTYPE1                1022
#define IDC_COMBO_PTYPE2                1023
#define IDC_PUBLISH                     1023
#define IDC_COMBO_PTYPE3                1024
#define IDC_EDIT2                       1024
#define IDC_PUBLISHIP                   1024
#define IDC_COMBO_PTYPE4                1025
#define IDC_COMBO_PTYPE5                1026
#define IDC_COMBO_PTYPE6                1027
#define IDC_COMBO_PTYPE7                1028
#define IDC_COMBO_PTYPE8                1029
#define IDC_COMBO_PTYPE9                1046
#define IDC_COMBO_PTYPE10               1047
#define IDC_COMBO_PTYPE11               1048
#define IDC_COMBO_PTYPE12               1049
#define IDC_COMBO_PTYPE13               1050
#define IDC_COMBO_PTYPE14               1051
#define IDC_COMBO_PTYPE15               1052
#define IDC_COMBO_PTYPE16               1053
#define IDC_COMBO_PTYPE17               1054
#define IDC_COMBO_PTYPE18               1055
#define IDC_COMBO_PTYPE19               1056
#define IDC_COMBO_PTYPE20               1057
#define IDC_COMBO_PTYPE21               1058
#define IDC_COMBO_PTYPE22               1059
#define IDC_COMBO_PTYPE23               1060
#define IDC_COMBO_PTYPE24               1061
#define IDC_DISPLAY                     2000
#define IDC_NOEXT                       2004
#define IDC_NOSOUND                     2007
#define IDC_NOMULTITEX                  2008
#define IDC_RADIO_SMALL                 2010
#define IDC_RADIO_MEDIUM                2011
#define IDC_STATIC_LOCATION             2012
#define IDC_RADIO_LARGE                 2013
#define IDC_FULLSCREEN                  2014
#define IDC_NOSKINS                     2015
#define IDC_NOWATERMOVE                 2016
#define IDC_NOSKYMOVE                   2017
#define IDC_FULLCLEAR                   2018
#define IDC_NOSKYLAYERS                 2019
#define IDC_JOINSERVER                  2020
#define IDC_EDIT_SERVER                 2021
#define IDC_SLIDER1                     2022
#define IDC_COMBO1                      2023
#define IDC_SERVER_LIST                 2024
#define IDC_BUTTON_LAN                  2025
#define IDC_BUTTON_NET                  2026
#define IDC_BUTTON4                     2028
#define IDC_PROGRESS1                   2029
#define IDC_BUTTON1                     2032
#define IDC_BUTTON_SCORCHED             2032
#define IDC_BUTTON_EYE                  2032
#define IDC_BUTTON3                     2034
#define IDC_BUTTON_STARTSINGLE          2034
#define IDC_BUTTON5                     2035
#define IDC_CLEAR                       2035
#define IDC_BUTTON_QUICKSINGLE          2035
#define IDC_BUTTON6                     2036
#define IDC_BUTTON_STARTNET             2036
#define IDC_BUTTON7                     2037
#define IDC_BUTTON_QUICKNET             2037
#define IDC_RADIO_SINGLE                2038
#define IDC_RADIO_SINGLEQUICK           2039
#define IDC_RADIO_NET                   2040
#define IDC_RADIO_NETQUICK              2041
#define IDC_RADIO5                      2042
#define IDC_RADIO_SERVER                2042
#define IDC_BUTTON2                     2042
#define IDC_BUTTON_SETTINGS             2042
#define IDC_EDIT3                       2043
#define IDC_STATIC                      -1

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NO_MFC                     1
#define _APS_NEXT_RESOURCE_VALUE        244
#define _APS_NEXT_COMMAND_VALUE         32771
#define _APS_NEXT_CONTROL_VALUE         2044
#define _APS_NEXT_SYMED_VALUE           210
#endif
#endif
