////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// WARNING: This was autogenerated from a windows .rc file!!
// Do not edit directly

#include "SettingsEnv-def.cpp"

static void createControls(wxWindow *parent)
{
	new wxStaticText(parent, -1,
		"Wind Force :",
		wxPoint((int) 84, (int) 30), wxSize((int) 63, (int) 12));
	new wxStaticText(parent, -1,
		"Wind Changes :",
		wxPoint((int) 69, (int) 52.5), wxSize((int) 78, (int) 12));
	IDC_COMBO_FORCE_CTRL = 
		new wxComboBox(parent, IDC_COMBO_FORCE,
		"",
		wxPoint((int) 154.5, (int) 28.5), wxSize((int) 132, (int) 91.5),
		0, 0, wxCB_READONLY);
	IDC_COMBO_WINDCHANGES_CTRL = 
		new wxComboBox(parent, IDC_COMBO_WINDCHANGES,
		"",
		wxPoint((int) 154.5, (int) 49.5), wxSize((int) 132, (int) 91.5),
		0, 0, wxCB_READONLY);
	IDC_COMBO_WALLTYPE_CTRL = 
		new wxComboBox(parent, IDC_COMBO_WALLTYPE,
		"",
		wxPoint((int) 154.5, (int) 70.5), wxSize((int) 132, (int) 91.5),
		0, 0, wxCB_READONLY);
	IDC_COMBO_WEAPONSCALE_CTRL = 
		new wxComboBox(parent, IDC_COMBO_WEAPONSCALE,
		"",
		wxPoint((int) 154.5, (int) 91.5), wxSize((int) 132, (int) 91.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Weapon Scale :",
		wxPoint((int) 70.5, (int) 93), wxSize((int) 76.5, (int) 12));
	new wxStaticText(parent, -1,
		"Wall Type :",
		wxPoint((int) 91.5, (int) 73.5), wxSize((int) 55.5, (int) 12));
	new wxStaticBox(parent, -1,
		"Env",
		wxPoint((int) 10.5, (int) 10.5), wxSize((int) 339, (int) 133.5));
	IDC_COMBO_ARMSLEVEL_CTRL = 
		new wxComboBox(parent, IDC_COMBO_ARMSLEVEL,
		"",
		wxPoint((int) 154.5, (int) 112.5), wxSize((int) 132, (int) 91.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Arms Level :",
		wxPoint((int) 87, (int) 114), wxSize((int) 60, (int) 12));
}

