////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// WARNING: This was autogenerated from a windows .rc file!!
// Do not edit directly

#include "SettingsPlayers-def.cpp"

static void createControls(wxWindow *parent)
{
	new wxStaticText(parent, -1,
		"Min Players :",
		wxPoint((int) 24, (int) 28.5), wxSize((int) 63, (int) 12));
	IDC_SERVER_MIN_PLAYERS_CTRL = 
		new wxComboBox(parent, IDC_SERVER_MIN_PLAYERS,
		"",
		wxPoint((int) 96, (int) 27), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	IDC_SERVER_MAX_PLAYERS_CTRL = 
		new wxComboBox(parent, IDC_SERVER_MAX_PLAYERS,
		"",
		wxPoint((int) 262.5, (int) 25.5), wxSize((int) 72, (int) 135),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Max Players :",
		wxPoint((int) 189, (int) 28.5), wxSize((int) 66, (int) 12));
	new wxStaticBox(parent, -1,
		"Players",
		wxPoint((int) 10.5, (int) 10.5), wxSize((int) 339, (int) 249));
	new wxStaticText(parent, -1,
		"1",
		wxPoint((int) 21, (int) 58.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE1_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE1,
		"",
		wxPoint((int) 33, (int) 54), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"2",
		wxPoint((int) 21, (int) 78), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE2_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE2,
		"",
		wxPoint((int) 33, (int) 75), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"3",
		wxPoint((int) 21, (int) 100.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE3_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE3,
		"",
		wxPoint((int) 33, (int) 96), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"4",
		wxPoint((int) 21, (int) 121.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE4_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE4,
		"",
		wxPoint((int) 33, (int) 117), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"5",
		wxPoint((int) 21, (int) 142.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE5_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE5,
		"",
		wxPoint((int) 33, (int) 138), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"6",
		wxPoint((int) 21, (int) 162), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE6_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE6,
		"",
		wxPoint((int) 33, (int) 159), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"7",
		wxPoint((int) 21, (int) 183), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE7_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE7,
		"",
		wxPoint((int) 33, (int) 180), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"8",
		wxPoint((int) 21, (int) 204), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE8_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE8,
		"",
		wxPoint((int) 33, (int) 201), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"9",
		wxPoint((int) 132, (int) 58.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE9_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE9,
		"",
		wxPoint((int) 144, (int) 55.5), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"10",
		wxPoint((int) 126, (int) 79.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE10_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE10,
		"",
		wxPoint((int) 144, (int) 75), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"11",
		wxPoint((int) 126, (int) 102), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE11_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE11,
		"",
		wxPoint((int) 144, (int) 96), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"12",
		wxPoint((int) 126, (int) 123), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE12_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE12,
		"",
		wxPoint((int) 144, (int) 117), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"13",
		wxPoint((int) 126, (int) 142.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE13_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE13,
		"",
		wxPoint((int) 144, (int) 138), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"14",
		wxPoint((int) 126, (int) 163.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE14_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE14,
		"",
		wxPoint((int) 144, (int) 159), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"15",
		wxPoint((int) 126, (int) 183), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE15_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE15,
		"",
		wxPoint((int) 144, (int) 180), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"16",
		wxPoint((int) 126, (int) 205.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE16_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE16,
		"",
		wxPoint((int) 144, (int) 202.5), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"17",
		wxPoint((int) 244.5, (int) 58.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE17_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE17,
		"",
		wxPoint((int) 262.5, (int) 55.5), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"18",
		wxPoint((int) 244.5, (int) 79.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE18_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE18,
		"",
		wxPoint((int) 262.5, (int) 75), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"19",
		wxPoint((int) 244.5, (int) 102), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE19_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE19,
		"",
		wxPoint((int) 262.5, (int) 96), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"20",
		wxPoint((int) 244.5, (int) 123), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE20_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE20,
		"",
		wxPoint((int) 262.5, (int) 117), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"21",
		wxPoint((int) 244.5, (int) 142.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE21_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE21,
		"",
		wxPoint((int) 262.5, (int) 138), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"22",
		wxPoint((int) 244.5, (int) 163.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE22_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE22,
		"",
		wxPoint((int) 262.5, (int) 159), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"23",
		wxPoint((int) 244.5, (int) 183), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE23_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE23,
		"",
		wxPoint((int) 262.5, (int) 180), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"24",
		wxPoint((int) 244.5, (int) 205.5), wxSize((int) 12, (int) 12));
	IDC_COMBO_PTYPE24_CTRL = 
		new wxComboBox(parent, IDC_COMBO_PTYPE24,
		"",
		wxPoint((int) 262.5, (int) 201), wxSize((int) 72, (int) 106.5),
		0, 0, wxCB_READONLY);
	IDC_EDIT3_CTRL = 
		new wxTextCtrl(parent, IDC_EDIT3,
		"",
		wxPoint((int) 118.5, (int) 229.5), wxSize((int) 214.5, (int) 21));
	new wxStaticText(parent, -1,
		"Bot Name Prefix :",
		wxPoint((int) 25.5, (int) 232.5), wxSize((int) 85.5, (int) 12));
}

