////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// GLWTextButton.h: interface for the GLWTextButton class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GLWTEXTBUTTON_H__36D49253_FFE6_4E93_89FC_F2CAC2DC3336__INCLUDED_)
#define AFX_GLWTEXTBUTTON_H__36D49253_FFE6_4E93_89FC_F2CAC2DC3336__INCLUDED_


#include <GLW/GLWButton.h>
#include <GLW/GLWLabel.h>

class GLWTextButton : public GLWButton
{
public:
	GLWTextButton(char *buttonText, float x, float y, float w, GLWButtonI *handler = 0,
		bool ok = false, bool cancel = false);
	virtual ~GLWTextButton();

	virtual void draw();

	void setText(const char *text) { label_.setText(text); }
	const char *getText() { return label_.getText(); }

METACLASSID

protected:
	GLWLabel label_;

};

#endif // !defined(AFX_GLWTEXTBUTTON_H__36D49253_FFE6_4E93_89FC_F2CAC2DC3336__INCLUDED_)
