////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// HeightMapModifier.h: interface for the HeightMapModifier class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HEIGHTMAPMODIFIER_H__26733049_91D4_4AB9_8870_1BC94F531E9A__INCLUDED_)
#define AFX_HEIGHTMAPMODIFIER_H__26733049_91D4_4AB9_8870_1BC94F531E9A__INCLUDED_

#include <common/RandomGenerator.h>
#include <landscape/HeightMap.h>

namespace HeightMapModifier  
{
	void levelSurround(HeightMap &hmap);
	void smooth(HeightMap &hmap, ProgressCounter *counter = 0);
	void scale(HeightMap &hmap, int maxHeight, RandomGenerator &generator,
			   ProgressCounter *counter = 0);

	void generateTerrain(HeightMap &hmap, int noHills, int maxHeight, 
						 int widthx, int widthy,
						 RandomGenerator &generator, 
						 RandomGenerator &offsetGenerator,
						 ProgressCounter *counter = 0);
	void addCirclePeak(HeightMap &hmap, Vector &start, 
					   float sizew, float sizew2, float sizeh,
					   RandomGenerator &generator);
};

#endif // !defined(AFX_HEIGHTMAPMODIFIER_H__26733049_91D4_4AB9_8870_1BC94F531E9A__INCLUDED_)
