////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


#ifndef YYSTYPE
#define YYSTYPE int
#endif
#define	MAX_DIGIT	258
#define	MAX_FLOAT	259
#define	MAX_STRING	260
#define	MAX_LBRACKET	261
#define	MAX_RBRACKET	262
#define	MAX_ASCIIEXPORT_SYM	263
#define	MAX_COMMENT	264
#define	MAX_MESH_FACE_ID	265
#define	MAX_SCENE	266
#define	MAX_SCENE_TOKEN	267
#define	MAX_MATERIAL_LIST	268
#define	MAX_MATERIAL_LIST_TOKEN	269
#define	MAX_GEOMOBJECT	270
#define	MAX_NODE_NAME	271
#define	MAX_NODE_TM	272
#define	MAX_NODE_TM_TOKEN	273
#define	MAX_MESH	274
#define	MAX_TIMEVALUE	275
#define	MAX_MESH_NUMVERTEX	276
#define	MAX_MESH_NUMFACES	277
#define	MAX_MESH_VERTEX_LIST	278
#define	MAX_MESH_VERTEX	279
#define	MAX_MESH_FACE_LIST	280
#define	MAX_MESH_FACE	281
#define	MAX_MESH_SMOOTHING	282
#define	MAX_MESH_MTLID	283
#define	MAX_MESH_NUMTVERTEX	284
#define	MAX_MESH_NORMALS	285
#define	MAX_MESH_FACENORMAL	286
#define	MAX_MESH_VERTEXNORMAL	287
#define	MAX_PROP_MOTIONBLUR	288
#define	MAX_PROP_CASTSHADOW	289
#define	MAX_PROP_RECVSHADOW	290
#define	MAX_WIREFRAME_COLOR	291
#define	MAX_TM_ROW0	292
#define	MAX_TM_ROW1	293
#define	MAX_TM_ROW2	294
#define	MAX_TM_ROW3	295


extern YYSTYPE aselval;
