////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


enum
{
	IDC_SERVER_MIN_PLAYERS = 1,
	IDC_SERVER_MAX_PLAYERS,
	IDC_COMBO_PTYPE1,
	IDC_COMBO_PTYPE2,
	IDC_COMBO_PTYPE3,
	IDC_COMBO_PTYPE4,
	IDC_COMBO_PTYPE5,
	IDC_COMBO_PTYPE6,
	IDC_COMBO_PTYPE7,
	IDC_COMBO_PTYPE8,
	IDC_COMBO_PTYPE9,
	IDC_COMBO_PTYPE10,
	IDC_COMBO_PTYPE11,
	IDC_COMBO_PTYPE12,
	IDC_COMBO_PTYPE13,
	IDC_COMBO_PTYPE14,
	IDC_COMBO_PTYPE15,
	IDC_COMBO_PTYPE16,
	IDC_COMBO_PTYPE17,
	IDC_COMBO_PTYPE18,
	IDC_COMBO_PTYPE19,
	IDC_COMBO_PTYPE20,
	IDC_COMBO_PTYPE21,
	IDC_COMBO_PTYPE22,
	IDC_COMBO_PTYPE23,
	IDC_COMBO_PTYPE24,
	IDC_EDIT3
};

static wxComboBox *IDC_SERVER_MIN_PLAYERS_CTRL = 0;
static wxComboBox *IDC_SERVER_MAX_PLAYERS_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE1_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE2_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE3_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE4_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE5_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE6_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE7_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE8_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE9_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE10_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE11_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE12_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE13_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE14_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE15_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE16_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE17_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE18_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE19_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE20_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE21_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE22_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE23_CTRL = 0;
static wxComboBox *IDC_COMBO_PTYPE24_CTRL = 0;
static wxTextCtrl *IDC_EDIT3_CTRL = 0;
