////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// WARNING: This was autogenerated from a windows .rc file!!
// Do not edit directly

#include "SettingsEco-def.cpp"

static void createControls(wxWindow *parent)
{
	new wxStaticText(parent, -1,
		"Start Money :",
		wxPoint((int) 112.5, (int) 33));
	IDC_STARTMONEY_CTRL = 
		new wxComboBox(parent, IDC_STARTMONEY,
		"",
		wxPoint((int) 184.5, (int) 30), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Percentage Interest :",
		wxPoint((int) 75, (int) 54));
	IDC_INTEREST_CTRL = 
		new wxComboBox(parent, IDC_INTEREST,
		"",
		wxPoint((int) 184.5, (int) 54), wxSize((int) 72, (int) 148.5),
		0, 0, wxCB_READONLY);
	new wxStaticBox(parent, -1,
		"Eco",
		wxPoint((int) 10.5, (int) 10.5), wxSize((int) 339, (int) 215.5));
	new wxStaticText(parent, -1,
		"Buy On Round :",
		wxPoint((int) 102, (int) 78));
	IDC_BUYONROUND_CTRL = 
		new wxComboBox(parent, IDC_BUYONROUND,
		"",
		wxPoint((int) 184.5, (int) 78), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Money Per hit Point :",
		wxPoint((int) 76, (int) 102));
	IDC_MONEYPERHIT_CTRL = 
		new wxComboBox(parent, IDC_MONEYPERHIT,
		"",
		wxPoint((int) 184.5, (int) 102), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Money Per kill Point :",
		wxPoint((int) 74, (int) 126));
	IDC_MONEYPERKILL_CTRL = 
		new wxComboBox(parent, IDC_MONEYPERKILL,
		"",
		wxPoint((int) 184.5, (int) 126), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Money Per Round Won :",
		wxPoint((int) 61.5, (int) 150));
	IDC_MONEYPERROUND_CTRL = 
		new wxComboBox(parent, IDC_MONEYPERROUND,
		"",
		wxPoint((int) 184.5, (int) 150), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	new wxStaticText(parent, -1,
		"Scoring Mode :",
		wxPoint((int) 61.5, (int) 174));
	IDC_SCOREMODE_CTRL = 
		new wxComboBox(parent, IDC_SCOREMODE,
		"",
		wxPoint((int) 184.5, (int) 174), wxSize((int) 72, (int) 124.5),
		0, 0, wxCB_READONLY);
	IDC_MONEYPERHEALTH_CTRL = 
		new wxCheckBox(parent, IDC_MONEYPERHEALTH,
		"Money Awarded by Health Taken",
		wxPoint((int) 41.5, (int) 198));
}

