////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////


// GameStateI.cpp: implementation of the GameStateI class.
//
//////////////////////////////////////////////////////////////////////

#include <engine/GameStateI.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GameStateI::~GameStateI()
{

}

void GameStateI::simulate(const unsigned state, float simTime)
{

}

void GameStateI::draw(const unsigned state)
{

}

void GameStateI::keyboardCheck(const unsigned state, float frameTime, 
							   char *buffer, unsigned int keyState,
							   KeyboardHistory::HistoryElement *, int hcount, 
							   bool &skipRest)
{

}

void GameStateI::mouseDown(const unsigned state, GameState::MouseButton button, 
						   int x, int y, bool &skipRest)
{

}

void GameStateI::mouseUp(const unsigned state, GameState::MouseButton button, 
						 int x, int y, bool &skipRest)
{

}

void GameStateI::mouseDrag(const unsigned state, GameState::MouseButton button, 
						   int mx, int my, int x, int y, bool &skipRest)
{

}

void GameStateI::mouseWheel(const unsigned state, short z, bool &skipRest)
{

}

void GameStateI::enterState(const unsigned state)
{

}
