////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////

#if !defined(__INCLUDE_ServerBrowserRefreshh_INCLUDE__)
#define __INCLUDE_ServerBrowserRefreshh_INCLUDE__

#include <client/ServerBrowserServerList.h>
#include <coms/NetServer.h>
#include <map>
#include <time.h>

class ServerBrowserRefresh
{
public:
	ServerBrowserRefresh(ServerBrowserServerList &list);
	virtual ~ServerBrowserRefresh();

	void refreshList();

protected:
	ServerBrowserServerList &list_;
	std::list<ServerBrowserEntry *> refreshEntries_;
	std::map<UDPsocket, ServerBrowserEntry*> entryMap_;
	UDPpacket *sendPacketStatus_;
	UDPpacket *sendPacketPlayers_;
	UDPpacket *recvPacket_;

	void sendNextEntry(ServerBrowserEntry *entry, time_t theTime);
	void processMessages(time_t theTime);
	void processMessage(UDPpacket *packet, ServerBrowserEntry *entry);
};

#endif
