////////////////////////////////////////////////////////////////////////////////
//    Scorched3D (c) 2000-2003
//
//    This file is part of Scorched3D.
//
//    Scorched3D is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Scorched3D is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Scorched3D; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
////////////////////////////////////////////////////////////////////////////////

static wxCheckBox *IDC_NOCOMPILEDARRAYS_CTRL = 0;
static wxCheckBox *IDC_NOENVCOMBINE_CTRL = 0;
static wxCheckBox *IDC_NOCUBEMAP_CTRL = 0;
static wxCheckBox *IDC_NOMIPMAPS_CTRL = 0;
static wxCheckBox *IDC_FULLCLEAR_CTRL = 0;
static wxCheckBox *IDC_NOEXT_CTRL = 0;
static wxCheckBox *IDC_NOSOUND_CTRL = 0;
static wxCheckBox *IDC_NOWATER_CTRL = 0;
static wxCheckBox *IDC_NOMULTITEX_CTRL = 0;
static wxCheckBox *IDC_INVERT_CTRL = 0;
static wxCheckBox *IDC_TIMER_CTRL = 0;
static wxCheckBox *IDC_NODETAILTEX_CTRL = 0;
static wxCheckBox *IDC_NOLANDSCAPESCORCH_CTRL = 0;
static wxRadioButton *IDC_SMALLTEX_CTRL = 0;
static wxRadioButton *IDC_MEDIUMTEX_CTRL = 0;
static wxRadioButton *IDC_LARGETEX_CTRL = 0;
static wxRadioButton *IDC_LOWTANK_CTRL = 0;
static wxRadioButton *IDC_MEDIUMTANK_CTRL = 0;
static wxRadioButton *IDC_HIGHTANK_CTRL = 0;
static wxRadioButton *IDC_LOWEFFECTS_CTRL = 0;
static wxRadioButton *IDC_MEDIUMEFFECTS_CTRL = 0;
static wxRadioButton *IDC_HIGHEFFECTS_CTRL = 0;
static wxCheckBox *IDC_NOSKINS_CTRL = 0;
static wxCheckBox *IDC_FULLSCREEN_CTRL = 0;
static wxComboBox *IDC_DISPLAY_CTRL = 0;
static wxSlider *IDC_SLIDER1_CTRL = 0;
static wxCheckBox *IDC_SINGLESKYLAYER_CTRL = 0;
static wxCheckBox *IDC_NOSKYANI_CTRL = 0;
static wxCheckBox *IDC_NOWATERANI_CTRL = 0;
static wxTextCtrl *IDC_USERID_CTRL = 0;
static wxButton *IDOK_CTRL = 0;
static wxButton *IDCANCEL_CTRL = 0;
