// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#define APSTUDIO_HIDDEN_SYMBOLS
#include "windows.h"
#undef APSTUDIO_HIDDEN_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDC_LAUNCHER MENU 
BEGIN
    POPUP "&File"
    BEGIN
        MENUITEM "Save as Defaults",            ID_FILE_SAVEASDEFAULTS
        MENUITEM "Restore Defaults",            ID_FILE_RESTOREDEFAULTS
        MENUITEM SEPARATOR
        MENUITEM "E&xit",                       IDM_EXIT
    END
    POPUP "&Help"
    BEGIN
        MENUITEM "&About ...",                  IDM_ABOUT
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// Accelerator
//

IDC_LAUNCHER ACCELERATORS 
BEGIN
    "?",            IDM_ABOUT,              ASCII,  ALT
    "/",            IDM_ABOUT,              ASCII,  ALT
END


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#define APSTUDIO_HIDDEN_SYMBOLS\r\n"
    "#include ""windows.h""\r\n"
    "#undef APSTUDIO_HIDDEN_SYMBOLS\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// Neutral (Sys. Default) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_NEUSD)
#ifdef _WIN32
LANGUAGE LANG_NEUTRAL, SUBLANG_SYS_DEFAULT
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_LAUNCHER_DIALOG DIALOGEX 0, 0, 359, 235
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | DS_CENTER | WS_MINIMIZEBOX | 
    WS_CAPTION | WS_SYSMENU
CAPTION "Scorched 3D"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Start a single or multi player game against the computer.\nYou must choose all the options.",
                    IDC_STATIC,162,103,184,17
    LTEXT           "Join a multiplayer game against other players over the internet.",
                    IDC_STATIC,162,152,161,17
    PUSHBUTTON      "Button1",IDC_BUTTON_SCORCHED,7,7,345,43,BS_BITMAP
    CONTROL         "Single/Multi Player Game",IDC_RADIO_SINGLE,"Button",
                    BS_AUTORADIOBUTTON,43,107,94,10
    CONTROL         "NET Game",IDC_RADIO_NET,"Button",BS_AUTORADIOBUTTON,43,
                    155,49,10
    LTEXT           "Start a single player game against the computer.\nStart straight away using the default options.",
                    IDC_STATIC,161,76,161,17
    CONTROL         "Quick Start Single Player Game",IDC_RADIO_SINGLEQUICK,
                    "Button",BS_AUTORADIOBUTTON,43,79,114,10
    LTEXT           "Start a server to allow clients to connect over a LAN or the internet.",
                    IDC_STATIC,161,176,161,17
    CONTROL         "Start A Server",IDC_RADIO_SERVER,"Button",
                    BS_AUTORADIOBUTTON,43,180,62,10
    CONTROL         "Quick Start NET Game",IDC_RADIO_NETQUICK,"Button",
                    BS_AUTORADIOBUTTON,43,130,86,10
    LTEXT           "Join a multiplayer game against other players over the internet.  Chooses the first available server.",
                    IDC_STATIC,161,126,161,17
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO 
BEGIN
    IDD_LAUNCHER_DIALOG, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 352
        TOPMARGIN, 7
        BOTTOMMARGIN, 227
    END
END
#endif    // APSTUDIO_INVOKED

#endif    // Neutral (Sys. Default) resources
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// English (U.K.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENG)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_UK
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDR_MENU1 MENU 
BEGIN
    POPUP "&File"
    BEGIN
        MENUITEM "E&xit",                       ID_FILE_EXIT
    END
    POPUP "&Settings"
    BEGIN
        MENUITEM "&View ASE Attributes",        ID_SETTINGS_VIEWASEATTRIBUTES

        MENUITEM "&Save Settings",              ID_SETTINGS_SAVESETTINGS
    END
    POPUP "&Player"
    BEGIN
        MENUITEM "&Kick",                       ID_PLAYER_KICK
        MENUITEM "&Talk To All",                ID_PLAYER_TALKTOALL
        MENUITEM "&Refresh Info",               ID_PLAYER_REFRESHINFO
    END
    POPUP "&Help"
    BEGIN
        MENUITEM "&About",                      ID_HELP_ABOUT
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_LAUNCHERFLAG_DIALOG DIALOGEX 0, 0, 360, 233
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | DS_CENTER | WS_POPUP | 
    WS_CAPTION | WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    CONTROL         "Full Clear - Clears screen instead of overwriting for each frame (slower)",
                    IDC_FULLCLEAR,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,19,
                    100,247,12
    GROUPBOX        "Troubleshooting Switches",IDC_STATIC,7,90,346,76
    CONTROL         "No GL Extensions - Don't use any OpenGL extensions (3dfx)",
                    IDC_NOEXT,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,19,113,
                    247,10
    CONTROL         "No Sound - Don't use any sound",IDC_NOSOUND,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,19,140,247,8
    CONTROL         "No Multi Texture - Only use one texture per face (faster)",
                    IDC_NOMULTITEX,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,19,
                    127,247,10
    GROUPBOX        "Texture Sizes (Larger is slower but of better quality)",
                    IDC_STATIC,7,7,346,29
    CONTROL         "Small (3dfx, fastest)",IDC_RADIO_SMALL,"Button",
                    BS_AUTORADIOBUTTON,44,20,81,10
    CONTROL         "Medium",IDC_RADIO_MEDIUM,"Button",BS_AUTORADIOBUTTON,
                    165,20,40,10
    CONTROL         "Large (slowest)",IDC_RADIO_LARGE,"Button",
                    BS_AUTORADIOBUTTON,258,21,63,10
    CONTROL         "No tank skins - Don't use texture for tanks (faster)",
                    IDC_NOSKINS,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,19,151,
                    274,12
    GROUPBOX        "Detail",IDC_STATIC,7,39,346,47
    CONTROL         "Don't animate the water",IDC_NOWATERMOVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,20,49,274,12
    CONTROL         "Don't animate the sky",IDC_NOSKYMOVE,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,20,60,274,12
    CONTROL         "Single sky layer",IDC_NOSKYLAYERS,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,20,71,274,12
    GROUPBOX        "Display",IDC_STATIC,8,170,345,51
    CONTROL         "Full Screen",IDC_FULLSCREEN,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,16,202,47,12
    COMBOBOX        IDC_DISPLAY,133,181,201,86,CBS_DROPDOWNLIST | WS_VSCROLL | 
                    WS_TABSTOP
    LTEXT           "Brightness :",IDC_STATIC,87,203,39,8
    CONTROL         "",IDC_SLIDER1,"msctls_trackbar32",TBS_AUTOTICKS | 
                    TBS_TOP | WS_TABSTOP,128,196,210,22
    LTEXT           "Resolution :",IDC_STATIC,87,185,39,8
END

IDD_LAUNCHERNET_DIALOG DIALOGEX 0, 0, 358, 265
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Server browsing brought to you by The All-Seeing Eye : http://www.udpsoft.com/eye",
                    IDC_STATIC,210,18,119,26
    CONTROL         "",IDC_SERVER_LIST,"SysListView32",LVS_REPORT | 
                    LVS_ALIGNLEFT | WS_BORDER | WS_TABSTOP,7,73,344,141
    PUSHBUTTON      "Refresh LAN Games",IDC_BUTTON_LAN,177,219,89,14
    PUSHBUTTON      "Refresh Internet Games",IDC_BUTTON_NET,269,219,82,14
    LTEXT           "Connect To Name/IP : ",IDC_STATIC,25,59,74,8
    CONTROL         "",IDC_PROGRESS1,"msctls_progress32",WS_BORDER,6,223,166,
                    7
    EDITTEXT        IDC_EDIT_SERVER,98,56,190,14,ES_AUTOHSCROLL
    PUSHBUTTON      "Button1",IDC_BUTTON_EYE,36,7,165,46,BS_BITMAP
    PUSHBUTTON      "Clear",IDC_CLEAR,291,58,31,10
    PUSHBUTTON      "Join Game",IDOK,301,243,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,247,243,50,14
END

IDD_ABOUTBOX DIALOGEX 22, 17, 281, 83
STYLE DS_SETFONT | DS_MODALFRAME | WS_CAPTION | WS_SYSMENU
CAPTION "About Scorched 3D"
FONT 8, "System", 0, 0, 0x0
BEGIN
    LTEXT           "Scorched 3D",IDC_STATIC,5,47,176,8,SS_NOPREFIX
    LTEXT           "Copyright (C) Gavin Camp 2002",IDC_STATIC,5,57,168,8
    DEFPUSHBUTTON   "OK",IDOK,245,67,30,11,WS_GROUP
    GROUPBOX        "",IDC_STATIC,5,0,270,44
    LTEXT           "Visit http:\\\\www.scorched3d.co.uk for news and updates.",
                    IDC_STATIC,6,68,221,10
    CONTROL         134,IDC_STATIC,"Static",SS_BITMAP,12,8,256,32
END

IDD_DIALOG_SERVER DIALOGEX 0, 0, 465, 274
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | DS_CENTER | WS_MINIMIZEBOX | 
    WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "Scorched 3D Dedicated Server"
MENU IDR_MENU1
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    CONTROL         "",IDC_LIST2,"SysListView32",LVS_REPORT | LVS_ALIGNLEFT | 
                    WS_BORDER | WS_TABSTOP,7,7,451,170
    CONTROL         "",IDC_LIST3,"SysListView32",LVS_REPORT | LVS_ALIGNLEFT | 
                    WS_BORDER | WS_TABSTOP,7,181,451,79
END

IDD_DIALOG_SSERVER DIALOGEX 0, 0, 240, 154
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | DS_CENTER | WS_POPUP | 
    WS_CAPTION | WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    EDITTEXT        IDC_SERVER_NAME,74,17,151,14,ES_AUTOHSCROLL
    EDITTEXT        IDC_SERVER_PORT,74,33,48,14,ES_AUTOHSCROLL
    LTEXT           "Port Number :",IDC_STATIC,14,36,46,8
    GROUPBOX        "Server Name",IDC_STATIC,7,7,226,78
    LTEXT           "Server Name :",IDC_STATIC,13,20,47,8
    CONTROL         "Allow other internet and LAN users to see this server",
                    IDC_PUBLISH,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,14,52,
                    209,10
    EDITTEXT        IDC_PUBLISHIP,74,66,152,14,ES_AUTOHSCROLL
    LTEXT           "Published Name :",IDC_STATIC,14,68,56,8
    DEFPUSHBUTTON   "Start Server",IDOK,182,135,50,14
    PUSHBUTTON      "Edit Advanced Settings",IDC_BUTTON_SETTINGS,13,104,100,
                    14
    PUSHBUTTON      "Cancel",IDCANCEL,126,135,50,14
    GROUPBOX        "Advanced Settings",IDC_STATIC,7,88,226,40
END

IDD_DIALOG_SENV DIALOGEX 0, 0, 239, 103
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Wind Force :",IDC_STATIC,56,20,42,8
    LTEXT           "Wind Changes :",IDC_STATIC,46,35,52,8
    COMBOBOX        IDC_COMBO_FORCE,103,19,88,61,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_WINDCHANGES,103,33,88,61,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_WALLTYPE,103,47,88,61,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_COMBO_WEAPONSCALE,103,61,88,61,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Weapon Scale :",IDC_STATIC,47,62,51,8
    LTEXT           "Wall Type :",IDC_STATIC,61,49,37,8
    GROUPBOX        "Env",IDC_STATIC,7,7,226,89
    COMBOBOX        IDC_COMBO_ARMSLEVEL,103,75,88,61,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Arms Level :",IDC_STATIC,58,76,40,8
END

IDD_DIALOG_SECO DIALOGEX 0, 0, 240, 113
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Start Money :",IDC_STATIC,75,22,45,8
    COMBOBOX        IDC_STARTMONEY,123,20,48,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Percentage Interest :",IDC_STATIC,50,36,70,8
    COMBOBOX        IDC_INTEREST,123,36,48,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Eco",IDC_STATIC,7,7,226,99
    LTEXT           "Buy On Round :",IDC_STATIC,68,52,52,8
    COMBOBOX        IDC_BUYONROUND,123,52,48,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Money Per Hit Point :",IDC_STATIC,52,68,68,8
    COMBOBOX        IDC_MONEYPERHIT,123,68,48,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Money Per Round Won :",IDC_STATIC,41,84,79,8
    COMBOBOX        IDC_MONEYPERROUND,123,84,48,83,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
END

IDD_DIALOG_SLAND DIALOGEX 0, 0, 240, 89
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Land Coverage :",IDC_STATIC,62,36,54,8
    COMBOBOX        IDC_LANDCOVERAGE,121,34,48,83,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Land Height :",IDC_STATIC,72,50,44,8
    COMBOBOX        IDC_LANDHEIGHT,121,49,48,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Land",IDC_STATIC,7,7,226,75
END

IDD_TALKDIALOG DIALOGEX 0, 0, 275, 44
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Talk To Players"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,218,23,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,163,23,50,14
    EDITTEXT        IDC_EDIT_TALK,7,7,261,12,ES_AUTOHSCROLL
END

IDD_DIALOG_SPLAYERS DIALOGEX 0, 0, 240, 179
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "Min Players :",IDC_STATIC,16,19,42,8
    COMBOBOX        IDC_SERVER_MIN_PLAYERS,64,18,48,83,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_SERVER_MAX_PLAYERS,175,17,48,90,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "Max Players :",IDC_STATIC,126,19,44,8
    GROUPBOX        "Players",IDC_STATIC,7,7,226,166
    LTEXT           "1",IDC_STATIC,14,39,8,8
    COMBOBOX        IDC_COMBO_PTYPE1,22,36,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "2",IDC_STATIC,14,52,8,8
    COMBOBOX        IDC_COMBO_PTYPE2,22,50,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "3",IDC_STATIC,14,67,8,8
    COMBOBOX        IDC_COMBO_PTYPE3,22,64,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "4",IDC_STATIC,14,81,8,8
    COMBOBOX        IDC_COMBO_PTYPE4,22,78,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "5",IDC_STATIC,14,95,8,8
    COMBOBOX        IDC_COMBO_PTYPE5,22,92,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "6",IDC_STATIC,14,108,8,8
    COMBOBOX        IDC_COMBO_PTYPE6,22,106,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "7",IDC_STATIC,14,122,8,8
    COMBOBOX        IDC_COMBO_PTYPE7,22,120,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "8",IDC_STATIC,14,136,8,8
    COMBOBOX        IDC_COMBO_PTYPE8,22,134,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "9",IDC_STATIC,88,39,8,8
    COMBOBOX        IDC_COMBO_PTYPE9,96,37,48,71,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "10",IDC_STATIC,84,53,8,8
    COMBOBOX        IDC_COMBO_PTYPE10,96,50,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "11",IDC_STATIC,84,68,8,8
    COMBOBOX        IDC_COMBO_PTYPE11,96,64,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "12",IDC_STATIC,84,82,8,8
    COMBOBOX        IDC_COMBO_PTYPE12,96,78,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "13",IDC_STATIC,84,95,8,8
    COMBOBOX        IDC_COMBO_PTYPE13,96,92,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "14",IDC_STATIC,84,109,8,8
    COMBOBOX        IDC_COMBO_PTYPE14,96,106,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "15",IDC_STATIC,84,122,8,8
    COMBOBOX        IDC_COMBO_PTYPE15,96,120,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "16",IDC_STATIC,84,137,8,8
    COMBOBOX        IDC_COMBO_PTYPE16,96,135,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "17",IDC_STATIC,163,39,8,8
    COMBOBOX        IDC_COMBO_PTYPE17,175,37,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "18",IDC_STATIC,163,53,8,8
    COMBOBOX        IDC_COMBO_PTYPE18,175,50,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "19",IDC_STATIC,163,68,8,8
    COMBOBOX        IDC_COMBO_PTYPE19,175,64,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "20",IDC_STATIC,163,82,8,8
    COMBOBOX        IDC_COMBO_PTYPE20,175,78,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "21",IDC_STATIC,163,95,8,8
    COMBOBOX        IDC_COMBO_PTYPE21,175,92,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "22",IDC_STATIC,163,109,8,8
    COMBOBOX        IDC_COMBO_PTYPE22,175,106,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "23",IDC_STATIC,163,122,8,8
    COMBOBOX        IDC_COMBO_PTYPE23,175,120,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    LTEXT           "24",IDC_STATIC,163,137,8,8
    COMBOBOX        IDC_COMBO_PTYPE24,175,134,48,71,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    EDITTEXT        IDC_EDIT3,79,153,143,14,ES_AUTOHSCROLL
    LTEXT           "Bot Name Prefix :",IDC_STATIC,17,155,57,8
END

IDD_DIALOG_SMAIN DIALOGEX 0, 0, 242, 126
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Dialog"
FONT 8, "MS Shell Dlg", 400, 0, 0x1
BEGIN
    LTEXT           "No Rounds :",IDC_STATIC,19,20,40,8
    COMBOBOX        IDC_SERVER_ROUNDS,65,18,48,83,CBS_DROPDOWNLIST | 
                    CBS_SORT | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Game Options",IDC_STATIC,7,7,226,66
    COMBOBOX        IDC_WAIT_TIME,167,18,58,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Wait Time :",IDC_STATIC,130,20,37,8
    COMBOBOX        IDC_SHOT_TIME,167,35,58,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Shot Time :",IDC_STATIC,130,37,37,8
    COMBOBOX        IDC_IDLE_TIME,167,52,58,83,CBS_DROPDOWNLIST | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    LTEXT           "Idle Time :",IDC_STATIC,133,54,34,8
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO 
BEGIN
    IDD_LAUNCHERFLAG_DIALOG, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 353
        TOPMARGIN, 7
        BOTTOMMARGIN, 224
    END

    IDD_LAUNCHERNET_DIALOG, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 351
        TOPMARGIN, 7
        BOTTOMMARGIN, 257
    END

    IDD_DIALOG_SMAIN, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 235
        TOPMARGIN, 7
        BOTTOMMARGIN, 119
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Bitmap
//

IDB_BITMAP1             BITMAP                  "../data/windows/scorched.BMP"
IDB_BITMAP2             BITMAP                  "../data/windows/eye1.bmp"

/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_ICON1               ICON                    "../data/windows/tank2.ico"
#endif    // English (U.K.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

